@@ -7,7 +7,7 @@ use vars qw(
  $BEFORE $AFTER $KEY $VALUE $BYTES $DIRTY
 );
 
-$VERSION = .11;
+$VERSION = .17;
 $Debug = 0; # set to 1 for summary, 2 for debug output
 $STRUCT_SIZE = 240; # per cached elem bytes overhead, approximate
 $REF_SIZE    = 16;
@@ -106,41 +106,54 @@ sub TIEHASH {
 
     my $sync = exists($options->{WriteSync}) ? $options->{WriteSync} : 1;
 
-    my $self = bless { 
-	   # how many items to cache
-	   max_count=> $max_count, 
-	   
-	   # max bytes to cache
-	   max_bytes => $options->{MaxBytes},
-	   
-	   # max size (in bytes) of an individual cache entry
-	   max_size => $options->{MaxSize} || (int($options->{MaxBytes}/10) + 1),
-	   
-	   # current sizes
-	   count=>0, 
-	   bytes=>0,
-	   
-	   # inner structures
-	   head=>0, 
-	   tail=>0, 
-	   nodes=>{},
-	   'keys'=>[],
-	   
-	   # statistics
-	   hit => 0,
-	   miss => 0,
-	   
-	   # config
-	   sync => $sync,
-	   dbg => $options->{Debug} || $Debug
-	   
-	
-    }, $class;
-
+    my $self = bless 
+      { 
+       # how many items to cache
+       max_count=> $max_count, 
+       
+       # max bytes to cache
+       max_bytes => $options->{MaxBytes},
+       
+       # max size (in bytes) of an individual cache entry
+       max_size => $options->{MaxSize} || ($options->{MaxBytes} ? (int($options->{MaxBytes}/10) + 1) : 0),
+       
+       # class track, so know if overridden subs should be used
+       'class'    => $class,
+       'subclass' => $class ne 'Tie::Cache' ? 1 : 0,
+       
+       # current sizes
+       count=>0,
+       bytes=>0,
+       
+       # inner structures
+       head=>0, 
+       tail=>0, 
+       nodes=>{},
+       'keys'=>[],
+       
+       # statistics
+       hit => 0,
+       miss => 0,
+       
+       # config
+       sync => $sync,
+       dbg => $options->{Debug} || $Debug
+       
+       
+      }, $class;
+    
     if (($self->{max_bytes} && ! $self->{max_size})) {
 	die("MaxSize must be defined when MaxBytes is");
     }
 
+    if($self->{max_bytes} and $self->{max_bytes} < 1000) {
+	die("cannot set MaxBytes to under 1000, each raw entry takes $STRUCT_SIZE bytes alone");
+    }
+
+    if($self->{max_size} && $self->{max_size} < 3) {
+	die("cannot set MaxSize to under 3 bytes, assuming error in config");
+    }
+
     $self;
 }
 
@@ -201,12 +214,15 @@ sub FETCH {
 	# that would then set the entry.  This model works well with
 	# sub-classing and reads() that might want to return undef as
 	# a valid value.
-	$self->print("read() for key $key") if $self->{dbg} > 1;
-	my $value = $self->read($key);
+	my $value;
+	if ($self->{subclass}) {
+	    $self->print("read() for key $key") if $self->{dbg} > 1;
+	    $value = $self->read($key);
+	}
 
 	if(defined $value) {
-	    my $length = 0;
-	    if(defined $self->{max_size}) {
+	    my $length;
+	    if($self->{max_size}) {
 		# check max size of entry, that it not exceed max size
 		$length = &_get_data_length(\$key, \$value);
 		if($length > $self->{max_size}) {
@@ -235,11 +251,13 @@ sub STORE {
 
     # check max size of entry, that it not exceed max size
     my $length;
-    if(defined($self->{max_size})) {
+    if($self->{max_size}) {
 	$length = &_get_data_length(\$key, \$value);
 	if($length > $self->{max_size}) {
-	    $self->print("direct write() [$key, $value]") if ($self->{dbg} > 1);
-	    $self->write($key, $value);	
+	    if ($self->{subclass}) {
+		$self->print("direct write() [$key, $value]") if ($self->{dbg} > 1);
+		$self->write($key, $value);
+	    }
 	    return $value;
 	}
     }
@@ -247,14 +265,9 @@ sub STORE {
     # do we have node already ?
     if($self->{nodes}{$key}) {
 	$node = &delete($self, $key);
-
-	## DON'T REUSE NOW, BETTER LENGTH REUSE
-#	$node->{value} = $value;
-	# update node bytes
-#	if ($self->{max_size}) {
-#	    $node->{bytes} = &_get_data_length(\$key, \$value);
-#	}
-
+#	$node = &delete($self, $key);
+#	$node->[$VALUE] = $value;
+#	$node->[$BYTES] = $length || &_get_data_length(\$key, \$value);
     }
 
     # insert new node  
@@ -264,11 +277,13 @@ sub STORE {
 
     # if the data is sync'd call write now, otherwise defer the data
     # writing, but mark it dirty so it can be cleanup up at the end
-    if($self->{sync}) {
-	$self->print("sync write() [$key, $value]") if $self->{dbg} > 1;
-	$self->write($key, $value);
-    } else {
-	$node->[$DIRTY] = 1;
+    if ($self->{subclass}) {
+	if($self->{sync}) {
+	    $self->print("sync write() [$key, $value]") if $self->{dbg} > 1;
+	    $self->write($key, $value);
+	} else {
+	    $node->[$DIRTY] = 1;
+	}
     }
 
     $value;
@@ -276,26 +291,25 @@ sub STORE {
 
 sub DELETE {
     my($self, $key) = @_;
-    
+
     $self->print("DELETE $key") if ($self->{dbg} > 1);
     my $node = $self->delete($key);
-    my $value = $node->[$VALUE];
-    undef $node;
-
-    $value;
+    $node ? $node->[$VALUE] : undef;
 }
 
 sub CLEAR {
     my($self) = @_;
 
     $self->print("CLEAR CACHE") if ($self->{dbg} > 1);
+
+    if($self->{subclass}) {
+	my $flushed = $self->flush();
+	$self->print("FLUSH COUNT $flushed") if ($self->{dbg} > 1);
+    }
+
     my $node;
     while($node = $self->{head}) {
 	$self->delete($self->{head}[$KEY]);
-	if($node->[$DIRTY]) {
-	    $self->print("dirty write() [$node->[$KEY], $node->[$VALUE]]") if ($self->{dbg} > 1);
-	    $self->write($node->[$KEY], $node->[$VALUE]);
-	}
     }
 
     1;
@@ -445,10 +459,12 @@ sub insert {
 			 );
 	}
 	my $old_node = $self->delete($self->{head}[$KEY]);
-	if($old_node->[$DIRTY]) {
-	    $self->print("dirty write() [$old_node->[$KEY], $old_node->[$VALUE]]") 
-	      if ($self->{dbg} > 1);
-	    $self->write($old_node->[$KEY], $old_node->[$VALUE]);
+	if ($self->{subclass}) {
+	    if($old_node->[$DIRTY]) {
+		$self->print("dirty write() [$old_node->[$KEY], $old_node->[$VALUE]]") 
+		  if ($self->{dbg} > 1);
+		$self->write($old_node->[$KEY], $old_node->[$VALUE]);
+	    }
 	}
 #	if($self->{dbg} > 1) {
 #	    $self->print("after delete - bytes $self->{bytes}; count $self->{count}");
@@ -488,6 +504,27 @@ sub delete {
     $node;
 }
 
+sub flush {
+    my $self = shift;
+
+    $self->print("FLUSH CACHE") if ($self->{dbg} > 1);
+
+    my $node = $self->{head};
+    my $flush_count = 0;
+    while($node) {
+	if($node->[$DIRTY]) {
+	    $self->print("flush dirty write() [$node->[$KEY], $node->[$VALUE]]") 
+	      if ($self->{dbg} > 1);
+	    $self->write($node->[$KEY], $node->[$VALUE]);
+	    $node->[$DIRTY] = 0;
+	    $flush_count++;
+	}
+	$node = $node->[$AFTER];
+    }
+
+    $flush_count;
+}
+
 sub print {
     my($self, $msg) = @_;
     print "$self: $msg\n";
@@ -547,24 +584,21 @@ shown below.  Otherwise, just copy Cache.pm to $PERLLIB/site/Tie
 
 =head1 BENCMARKS
 
-There is another simpler LRU cache implementation in CPAN, 
+There is another simpler LRU cache implementation in CPAN,
 Tie::Cache::LRU, which has the same basic size limiting 
 functionality, and for this functionality, the exact same 
-interface.  This other cache takes writes about 20% faster 
-but cache reads are about 50% slower.  Here are some numbers 
-to illustrate:
+interface.
 
- Module			Read/s	Write/s	Delete/s Platform
- ------			------	-------	-------- --------
- Tie::Cache v.08	 6300	3100	4800	 perl 5.00404 WinNT PII300
- Tie::Cache::LRU v.05	 3700	3700	4500	 perl 5.00404 WinNT PII300
- --
- Tie::Cache v.08	10600	5300	8500	 perl 5.00503 Solaris PII300
+Through healthy competition, Michael G Schwern got 
+Tie::Cache::LRU mostly faster than Tie::Cache on reads & writes:
 
-The reason for using an cache is that you are probably
-doing more reads than writes, so you will likely want to 
-use this module, but may want to consider Tie::Cache::LRU
-if your i/o mix is write heavy.  
+ Cache Size 5000       Tie::Cache 0.17  Tie::Cache::LRU 0.21
+ 10000 Writes             1.55 CPU sec          1.10 CPU sec
+ 40000 Reads              1.82 CPU sec          1.58 CPU sec
+ 10000 Deletes            0.55 CPU sec          0.59 CPU sec
+
+Unless you are using TRUE CACHE or MaxBytes functionality,
+using Tie::Cache::LRU should be an easy replacement for Tie::Cache.
 
 =head1 TRUE CACHE
 
@@ -581,6 +615,14 @@ been modified in the cache gets written out when the entries are deleted or
 during the DESTROY phase of the cache object, usually at the end of
 a script.
 
+To have the dirty data write() periodically while WriteSync is set to 0,
+there is a flush() cache API call that will flush the dirty writes
+in this way.  Just call the flush() API like:
+
+  my $write_flush_count = tied(%cache)->flush();
+
+The flush() API was added in the .17 release thanks to Rob Bloodgood.
+
 =head1 TRUE CACHE EXAMPLE
 
  use Tie::Cache;
@@ -613,6 +655,9 @@ a script.
  while(my($k, $v) = each %cache) { print "each data $k: $v\n"; }
  for(my $i=$num_to_cache; $i>0; $i--) { print "read data $i: $cache{$i}\n"; }
 
+ # flush writes now, trivial use since will happen in DESTROY() anyway
+ tied(%cache)->flush(); 
+
  # clear cache in 2 ways, write will flush out to disk
  %cache = ();
  undef %cache;
@@ -627,6 +672,7 @@ including:
 	:) Jamie McCarthy, for trying to make Tie::Cache be all
 	   that it can be.
 	:) Rob Fugina who knows how to "TRULY CACHE".
+	:) Rob Bloodgood, for the TRUE CACHE flush() API
 
 =head1 AUTHOR
 
@@ -635,14 +681,13 @@ at chamas@alumni.stanford.org
 
 =head1 COPYRIGHT
 
-Copyright (c) 1999-2001 Joshua Chamas, Chamas Enterprises Inc.  
+Copyright (c) 1999-2002 Joshua Chamas, Chamas Enterprises Inc.  
+Sponsored by development on NodeWorks http://www.nodeworks.com
 
 All rights reserved. This program is free software; 
 you can redistribute it and/or modify it under the same 
 terms as Perl itself. 
 
-Sponsored by development on NodeWorks http://www.nodeworks.com
-
 =cut
 
 
